@@ -20,6 +20,7 @@ from ..common.models import (
     EventMessage,
     EventType,
     MatterNodeData,
+    MatterNodeEvent,
     MessageType,
     ResultMessageBase,
     ServerDiagnostics,
@@ -121,7 +122,10 @@ class MatterClient:
 
     async def commission_on_network(self, setup_pin_code: int) -> MatterNodeData:
         """
-        Commission a device already connected to the network.
+        Do the routine for OnNetworkCommissioning.
+
+        NOTE: For advanced usecases only, use `commission_with_code`
+        for regular commissioning.
 
         Returns basic MatterNodeData once complete.
         """
@@ -174,7 +178,7 @@ class MatterClient:
 
         node = self.get_node(node_id)
         fabrics: list[
-            Clusters.OperationalCredentials.Structs.FabricDescriptor
+            Clusters.OperationalCredentials.Structs.FabricDescriptorStruct
         ] = node.get_attribute_value(
             0, None, Clusters.OperationalCredentials.Attributes.Fabrics
         )
@@ -182,16 +186,16 @@ class MatterClient:
         vendors_map = await self.send_command(
             APICommand.GET_VENDOR_NAMES,
             require_schema=3,
-            filter_vendors=[f.vendorId for f in fabrics],
+            filter_vendors=[f.vendorID for f in fabrics],
         )
 
         return [
             MatterFabricData(
-                fabric_id=f.fabricId,
-                vendor_id=f.vendorId,
+                fabric_id=f.fabricID,
+                vendor_id=f.vendorID,
                 fabric_index=f.fabricIndex,
                 fabric_label=f.label if f.label else None,
-                vendor_name=vendors_map.get(str(f.vendorId)),
+                vendor_name=vendors_map.get(str(f.vendorID)),
             )
             for f in fabrics
         ]
@@ -486,6 +490,19 @@ class MatterClient:
             node_id = msg.data["node_id"]
             endpoint_id = msg.data["endpoint_id"]
             self.logger.debug("Endpoint added: %s/%s", node_id, endpoint_id)
+        if msg.event == EventType.NODE_EVENT:
+            if self.logger.isEnabledFor(logging.DEBUG):
+                self.logger.debug(
+                    "Node event: %s",
+                    msg.data,
+                )
+            node_event = dataclass_from_dict(MatterNodeEvent, msg.data)
+            self._signal_event(
+                EventType.NODE_EVENT,
+                data=node_event,
+                node_id=node_event.node_id,
+            )
+            return
         # simply forward all other events as-is
         if self.logger.isEnabledFor(logging.DEBUG):
             self.logger.debug("Received event: %s", msg)
