@@ -7,7 +7,8 @@ from collections import deque
 from datetime import datetime
 from functools import partial
 import logging
-from typing import TYPE_CHECKING, Any, Callable, Deque, Iterable, Type, TypeVar, cast
+import random
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Type, TypeVar, cast
 
 from chip.ChipDeviceCtrl import CommissionableNode
 from chip.clusters import Attribute, Objects as Clusters
@@ -35,7 +36,7 @@ from ..common.helpers.util import (
     dataclass_from_dict,
     parse_attribute_path,
 )
-from ..common.models import APICommand, EventType, MatterNodeData
+from ..common.models import APICommand, EventType, MatterNodeData, MatterNodeEvent
 from .const import PAA_ROOT_CERTS_DIR
 from .helpers.paa_certificates import fetch_certificates
 
@@ -50,7 +51,7 @@ DATA_KEY_NODES = "nodes"
 DATA_KEY_LAST_NODE_ID = "last_node_id"
 
 LOGGER = logging.getLogger(__name__)
-INTERVIEW_TASK_LIMIT = 5
+MAX_POLL_INTERVAL = 600
 
 # a list of attributes we should always watch on all nodes
 DEFAULT_SUBSCRIBE_ATTRIBUTES: set[tuple[int | str, int | str, int | str]] = {
@@ -73,17 +74,16 @@ class MatterDeviceController:
         """Initialize the device controller."""
         self.server = server
         # we keep the last events in memory so we can include them in the diagnostics dump
-        self.event_history: Deque[Attribute.EventReadResult] = deque(maxlen=25)
+        self.event_history: deque[Attribute.EventReadResult] = deque(maxlen=25)
         self._subscriptions: dict[int, Attribute.SubscriptionTransaction] = {}
         self._attr_subscriptions: dict[int, list[tuple[Any, ...]] | str] = {}
-        self._resub_timer: dict[int, asyncio.TimerHandle] = {}
+        self._resub_debounce_timer: dict[int, asyncio.TimerHandle] = {}
+        self._sub_retry_timer: dict[int, asyncio.TimerHandle] = {}
         self._nodes: dict[int, MatterNodeData | None] = {}
         self.wifi_credentials_set: bool = False
         self.thread_credentials_set: bool = False
         self.compressed_fabric_id: int | None = None
-        self._interview_limit: asyncio.Semaphore = asyncio.Semaphore(
-            INTERVIEW_TASK_LIMIT
-        )
+        self._resolve_lock: asyncio.Lock = asyncio.Lock()
         self._node_lock: dict[int, asyncio.Lock] = {}
 
     async def initialize(self) -> None:
@@ -188,11 +188,14 @@ class MatterDeviceController:
         filter: Any = None,  # pylint: disable=redefined-builtin
     ) -> MatterNodeData:
         """
-        Commission a device already connected to the network.
+        Do the routine for OnNetworkCommissioning, with a filter for mDNS discovery.
 
-        Does the routine for OnNetworkCommissioning, with a filter for mDNS discovery.
         The filter can be an integer,
         a string or None depending on the actual type of selected filter.
+
+        NOTE: For advanced usecases only, use `commission_with_code`
+        for regular commissioning.
+
         Returns full NodeInfo once complete.
         """
         if self.chip_controller is None:
@@ -303,16 +306,14 @@ class MatterDeviceController:
         LOGGER.debug("Interviewing node: %s", node_id)
         try:
             await self._resolve_node(node_id=node_id)
-            async with self._interview_limit:
-                async with self._get_node_lock(node_id):
-                    read_response: Attribute.AsyncReadTransaction.ReadResponse = (
-                        await self.chip_controller.Read(
-                            nodeid=node_id,
-                            attributes="*",
-                            events="*",
-                            fabricFiltered=False,
-                        )
+            async with self._get_node_lock(node_id):
+                read_response: Attribute.AsyncReadTransaction.ReadResponse = (
+                    await self.chip_controller.Read(
+                        nodeid=node_id,
+                        attributes="*",
+                        fabricFiltered=False,
                     )
+                )
         except (ChipStackError, NodeNotResolving) as err:
             raise NodeInterviewFailed(f"Failed to interview node {node_id}") from err
 
@@ -383,6 +384,50 @@ class MatterDeviceController:
                 interactionTimeoutMs=interaction_timeout_ms,
             )
 
+    @api_command(APICommand.READ_ATTRIBUTE)
+    async def read_attribute(
+        self,
+        node_id: int,
+        attribute_path: str,
+    ) -> Any:
+        """Read a single attribute on a node."""
+        if self.chip_controller is None:
+            raise RuntimeError("Device Controller not initialized.")
+        node_lock = self._get_node_lock(node_id)
+        await self._resolve_node(node_id=node_id)
+        endpoint_id, cluster_id, attribute_id = parse_attribute_path(attribute_path)
+        attribute: Type[ClusterAttributeDescriptor] = ALL_ATTRIBUTES[cluster_id][
+            attribute_id
+        ]
+        async with node_lock:
+            result: Attribute.AsyncReadTransaction.ReadResponse = (
+                await self.chip_controller.Read(
+                    nodeid=node_id,
+                    attributes=[(endpoint_id, attribute)],
+                    fabricFiltered=False,
+                )
+            )
+            read_atributes = self._parse_attributes_from_read_result(result.attributes)
+            return read_atributes.get(attribute_path, None)
+
+    @api_command(APICommand.WRITE_ATTRIBUTE)
+    async def write_attribute(
+        self,
+        node_id: int,
+        attribute_path: str,
+        value: Any,
+    ) -> Any:
+        """Write an attribute(value) on a target node."""
+        if self.chip_controller is None:
+            raise RuntimeError("Device Controller not initialized.")
+        endpoint_id, cluster_id, attribute_id = parse_attribute_path(attribute_path)
+        attribute = ALL_ATTRIBUTES[cluster_id][attribute_id]()
+        attribute.value = value
+        return await self.chip_controller.WriteAttribute(
+            nodeid=node_id,
+            attributes=[(endpoint_id, attribute)],
+        )
+
     @api_command(APICommand.REMOVE_NODE)
     async def remove_node(self, node_id: int) -> None:
         """Remove a Matter node/device from the fabric."""
@@ -394,6 +439,9 @@ class MatterDeviceController:
                 f"Node {node_id} does not exist or has not been interviewed."
             )
 
+        # pop any existing interview/subscription reschedule timer
+        self._sub_retry_timer.pop(node_id, None)
+
         node = self._nodes.pop(node_id)
         self.server.storage.remove(
             DATA_KEY_NODES,
@@ -459,13 +507,15 @@ class MatterDeviceController:
         # this could potentially be called multiple times within a short timeframe
         # so debounce it a bit
         def resubscribe() -> None:
-            self._resub_timer.pop(node_id, None)
+            self._resub_debounce_timer.pop(node_id, None)
             asyncio.create_task(self._subscribe_node(node_id))
 
-        if existing_timer := self._resub_timer.pop(node_id, None):
+        if existing_timer := self._resub_debounce_timer.pop(node_id, None):
             existing_timer.cancel()
         assert self.server.loop is not None
-        self._resub_timer[node_id] = self.server.loop.call_later(5, resubscribe)
+        self._resub_debounce_timer[node_id] = self.server.loop.call_later(
+            5, resubscribe
+        )
 
     async def _subscribe_node(self, node_id: int) -> None:
         """
@@ -515,16 +565,11 @@ class MatterDeviceController:
                 # Wildcard endpoint, specific cluster
                 attr_subscriptions.append(cluster)
 
-        if len(attr_subscriptions) > 50:
-            # prevent memory overload on node and fallback to wildcard sub if too many
-            # individual subscriptions (e.g. bridges)
-            attr_subscriptions = "*"  # type: ignore[assignment]
-
-        if not node.attribute_subscriptions:
-            # temp fix for backwards compatbility with HA releases below 2023.7
-            # fallback to wildcard subscriptions if we have no explicit
-            # node subscriptions defined.
-            # TODO: remove this after HA release 2023.8
+        if len(attr_subscriptions) > 9:
+            # strictly taken a matter device can only handle 9 individual subscriptions
+            # (3 subscriptions of 3 paths per fabric)
+            # although the device can probably handle more, we play it safe and opt for
+            # wildcard as soon as we have more than 9 paths to watch for.
             attr_subscriptions = "*"  # type: ignore[assignment]
 
         # check if we already have an subscription for this node,
@@ -539,23 +584,34 @@ class MatterDeviceController:
                 node_logger.debug("Unsubscribing from existing subscription.")
                 await self._call_sdk(prev_sub.Shutdown)
 
-        node_logger.debug("Setting up attributes and events subscription.")
         self._attr_subscriptions[node_id] = attr_subscriptions
         async with node_lock:
+            node_logger.debug("Setting up attributes and events subscription.")
+            # Use a report interval of 0, X which means we want to receive state changes
+            # as soon as possible (the 0 as floor) but we want to receive a report
+            # at least once every X seconds, this is also used to detect the node is still alive.
+            # A resubscription will be initiated automatically by the sdk
+            # if there was no report within the interval.
+            # NOTE 1: The report interval ceiling is subject to a lot of discussion
+            # as setting it too low causes a lot of (unneeded) traffic and causes network
+            # congestion as well as drains batteries on sleeping devices.
+            # Preferred would be to set the interval as high as possible ( 30 mins or even 1 hour)
+            # but that would also mean that detecting that a device is offline would be delayed
+            # by that amount of time as the interval ceiling also meant as liveness detection.
+            # For now we settle on (more or less) 10 minutes but we might need to increase this
+            # even more in the future.
+            # see also: https://github.com/project-chip/connectedhomeip/issues/29804
+            # NOTE 2: We randomize the interval a bit to prevent all nodes reporting
+            # at the exact same time, causing congestion.
+            interval_floor = 0
+            interval_ceiling = random.randint(500, 700)
             sub: Attribute.SubscriptionTransaction = await self.chip_controller.Read(
                 nodeid=node_id,
-                # In order to prevent network congestion due to wildcard subscriptions on all nodes,
-                # we keep a list of attributes we are explicitly interested in.
                 attributes=attr_subscriptions,
-                # simply subscribe to all (urgent and non urgent) device events
-                events=[("*", 1), ("*", 0)],
-                # Use a report interval of 0, 300 which means we want to receive state changes
-                # as soon as possible (the 0 as floor) but we want to receive a report
-                # at least once every 5 minutes (300 as ceiling).
-                # This is also used to detect the node is still alive.
-                # A resubscription will be initiated automatically by the sdk
-                # if there was no report within the interval.
-                reportInterval=(0, 300),
+                # simply subscribe to urgent device events only (e.g. button press etc.)
+                # non urgent events are diagnostic reports etc. for which we have no usecase (yet).
+                events=[("*", 1)],
+                reportInterval=(interval_floor, interval_ceiling),
                 # Use fabricfiltered as False to detect changes made by other controllers
                 # and to be able to provide a list of all fabrics attached to the device
                 fabricFiltered=False,
@@ -600,6 +656,14 @@ class MatterDeviceController:
                     )
                 return
 
+            # work out if software version changed
+            if (
+                path.AttributeType == Clusters.BasicInformation.softwareVersion
+                and new_value != old_value
+            ):
+                # schedule a full interview of the node if the software version changed
+                self.server.loop.create_task(self.interview_node(node_id))
+
             # store updated value in node attributes
             node.attributes[attr_path] = new_value
 
@@ -627,9 +691,20 @@ class MatterDeviceController:
             node_logger.debug(
                 "Received node event: %s - transaction: %s", data, transaction
             )
-            self.event_history.append(data)
+            node_event = MatterNodeEvent(
+                node_id=node_id,
+                endpoint_id=data.Header.EndpointId,
+                cluster_id=data.Header.ClusterId,
+                event_id=data.Header.EventId,
+                event_number=data.Header.EventNumber,
+                priority=data.Header.Priority,
+                timestamp=data.Header.Timestamp,
+                timestamp_type=data.Header.TimestampType,
+                data=data.Data,
+            )
+            self.event_history.append(node_event)
             self.server.loop.call_soon_threadsafe(
-                self.server.signal_event, EventType.NODE_EVENT, data
+                self.server.signal_event, EventType.NODE_EVENT, node_event
             )
 
         def error_callback(
@@ -653,6 +728,20 @@ class MatterDeviceController:
             if node.available:
                 node.available = False
                 self.server.signal_event(EventType.NODE_UPDATED, node)
+            if nextResubscribeIntervalMsec / 1000 > MAX_POLL_INTERVAL:
+                # workaround to handle devices that are unplugged
+                # from power for a longer period of time
+                # where the sdk is extending the poll timeout at every attempt
+                # until even 1,5 hours which is way too long.
+                # instead a device back alive should be detected using mDNS,
+                # which is not yet implemented in the core sdk.
+                # For now, we just override the timeout.
+                # NOTE 1: fix this once OperationalNodeDiscovery is available:
+                # https://github.com/project-chip/connectedhomeip/pull/26718
+                # https://github.com/project-chip/connectedhomeip/issues/29663
+                # NOTE 2: We could also just implement zeroconf/mdns ourselves
+                # to listen for the announcements.
+                sub.OverrideLivenessTimeoutMs(MAX_POLL_INTERVAL * 1000)
 
         def resubscription_succeeded(
             transaction: Attribute.SubscriptionTransaction,
@@ -700,20 +789,27 @@ class MatterDeviceController:
         )
 
     async def _check_interview_and_subscription(
-        self, node_id: int, reschedule_interval: int = 300
+        self, node_id: int, reschedule_interval: int = 30
     ) -> None:
         """Handle interview (if needed) and subscription for known node."""
 
+        if node_id not in self._nodes:
+            raise NodeNotExists(f"Node {node_id} does not exist.")
+
+        # pop any existing reschedule timer
+        self._sub_retry_timer.pop(node_id, None)
+
         def reschedule() -> None:
             """(Re)Schedule interview and/or initial subscription for a node."""
             assert self.server.loop is not None
-            self.server.loop.call_later(
+            self._sub_retry_timer[node_id] = self.server.loop.call_later(
                 reschedule_interval,
                 asyncio.create_task,
                 self._check_interview_and_subscription(
                     node_id,
-                    # increase interval at each attempt with maximum of 1 hour
-                    min(reschedule_interval + 300, 3600),
+                    # increase interval at each attempt with maximum of
+                    # MAX_POLL_INTERVAL seconds (= 10 minutes)
+                    min(reschedule_interval + 10, MAX_POLL_INTERVAL),
                 ),
             )
 
@@ -723,8 +819,6 @@ class MatterDeviceController:
             node_data is None
             # re-interview if the schema has changed
             or node_data.interview_version < SCHEMA_VERSION
-            # re-interview every 30 days
-            or (datetime.utcnow() - node_data.last_interview).days > 30
         ):
             try:
                 await self.interview_node(node_id)
@@ -749,6 +843,8 @@ class MatterDeviceController:
                 "will retry later in the background.",
                 node_id,
             )
+            # TODO: fix this once OperationalNodeDiscovery is available:
+            # https://github.com/project-chip/connectedhomeip/pull/26718
             reschedule()
 
     @staticmethod
@@ -793,37 +889,29 @@ class MatterDeviceController:
                     result[attribute_path] = attr_value
         return result
 
-    async def _resolve_node(
-        self, node_id: int, retries: int = 3, allow_pase: bool = False
-    ) -> None:
+    async def _resolve_node(self, node_id: int, retries: int = 3) -> None:
         """Resolve a Node on the network."""
+        if (node := self._nodes.get(node_id)) and node.available:
+            # no need to resolve, the node is already available/connected
+            return
+
         node_lock = self._get_node_lock(node_id)
         if self.chip_controller is None:
             raise RuntimeError("Device Controller not initialized.")
         try:
-            if allow_pase:
-                # last attempt allows PASE connection (last resort)
-                LOGGER.debug(
-                    "Attempting to resolve node %s (with PASE connection)", node_id
+            # the sdk crashes when multiple resolves happen at the same time
+            # guard simultane resolves with a lock.
+            async with node_lock, self._resolve_lock:
+                LOGGER.debug("Attempting to resolve node %s...", node_id)
+                await self._call_sdk(
+                    self.chip_controller.ResolveNode,
+                    nodeid=node_id,
                 )
-                async with node_lock:
-                    await self._call_sdk(
-                        self.chip_controller.GetConnectedDeviceSync,
-                        nodeid=node_id,
-                        allowPASE=True,
-                        timeoutMs=30000,
-                    )
-                return
-            LOGGER.debug("Resolving node %s", node_id)
-            await self._call_sdk(self.chip_controller.ResolveNode, nodeid=node_id)
         except (ChipStackError, TimeoutError) as err:
-            if not retries:
+            if retries <= 1:
                 # when we're out of retries, raise NodeNotResolving
                 raise NodeNotResolving(f"Unable to resolve Node {node_id}") from err
-            async with node_lock:
-                await self._resolve_node(
-                    node_id=node_id, retries=retries - 1, allow_pase=retries - 1 == 0
-                )
+            await self._resolve_node(node_id=node_id, retries=retries - 1)
             await asyncio.sleep(2)
 
     def _handle_endpoints_removed(self, node_id: int, endpoints: Iterable[int]) -> None:
